#ifndef CODEC_INTERFACE_H_INCLUDED
#define CODEC_INTERFACE_H_INCLUDED
#include "types.h"
#include "SPEEX\config.h"
#include "SPEEX\speex.h"
#include "SPEEX\speex_callbacks.h"
#include "SPEEX\speex_echo.h"
#include "SPEEX\speex_preprocess.h"

#include "waveio.h"


static const u32 ENCODED_PACKET_MAX_SIZE_BYTES = 200;

//------------- Mode definitions --------

#define MODE_16K  // Do not change this!
//#define MODE_8K

#define COMPLEXITY      3 // 1...10

//----------------------------------------

#ifdef MODE_8K
 #define FRAME_SIZE         160  // input samples per frame, 160 samples at 8K = 20ms
 #define QUALITY            4    // 11 kbit/s 
 #define EC_FILTER_LENGTH   1024 // about 100 ms
 #define SPEEX_SAMPLE_RATE  8000  
 #define FLAT_DELAY_SAMPLES (FRAME_SIZE*(WAVEOUT_NUM_OF_BUFFERS-1))
 
#endif

#ifdef MODE_16K
 #define FRAME_SIZE         320   // input samples per frame, 320 samples at 16K = 20ms
 #define QUALITY            5 // 16.8 kbit/s
 #define EC_FILTER_LENGTH   2048  // about 100 ms
 #define SPEEX_SAMPLE_RATE  16000
 #define FLAT_DELAY_SAMPLES (FRAME_SIZE*(WAVEOUT_NUM_OF_BUFFERS-1))
  
#endif



class SPEEX_CODEC
 {
 // Codec 
 void *st;
 void *dec;
 SpeexBits coder_bits;
 SpeexBits decoder_bits;

 // EC
 
 SpeexEchoState *echo_state;
 SpeexPreprocessState *preprocess_state;

 // Flat delay 

 s16 *delayed_spkr_data;
 s16 *flat_delay_buffer;
 u32 flat_delay_buffer_idx;
 u32 flat_delay_buffer_samples;

public:
    SPEEX_CODEC();
    ~SPEEX_CODEC();

    // CODEC functions
    u32 Pack_Audio_Data(u8 *packed_buffer, s16 * input_audio_data); // Returns the number of bytes
    void Unpack_Audio_Data(s16 *output_audio_data, u8 *packed_buffer,  u32 packed_data_bytes); // packet_buffer == NULL for lost frame

    // EC functions
    void Process_EC(s16 *output_audio_data, s16 *mic_audio_data, s16 *spkr_audio_data);

 };



#endif

