/*
Primitive functions for working with Win32 wave API
*/

#include "waveio.h"
#include "types.h"

//=========================================

void CALLBACK waveInProc(HWAVEIN hwi,UINT uMsg,DWORD dwInstance,DWORD dwParam1,DWORD dwParam2)
{
WAVEIN *wavein;

if(uMsg == WIM_DATA)
 {
 wavein = (WAVEIN *)dwInstance;
 
 EnterCriticalSection(&wavein->cswi);
 wavein->wavein_flag++;
 LeaveCriticalSection(&wavein->cswi);
 SetEvent(wavein->rxEvent);
 }
}     

//==============================================

WAVEIN::WAVEIN(UINT waveInDevID)
{
u32 i;

//==================== Initialize Wave Input ==============

InitializeCriticalSection(&cswi); 

waveformatex.wFormatTag = WAVE_FORMAT_PCM;
waveformatex.nChannels  = WAVEIN_CHANNELS;
waveformatex.nSamplesPerSec  = (int)WAVEIN_FS;
waveformatex.nAvgBytesPerSec = (int)(WAVEIN_FS*2*WAVEIN_CHANNELS);
waveformatex.nBlockAlign = 2*WAVEIN_CHANNELS;
waveformatex.wBitsPerSample = 16;
waveformatex.cbSize = 0;

for(i = 0; i < WAVEIN_NUM_OF_BUFFERS; i++)
 {
 wi_data[i] = new char[WAVEIN_BUFFER_LENGTH];
 memset(wi_data[i],0,WAVEIN_BUFFER_LENGTH);
 wi[i].lpData = wi_data[i];
 wi[i].dwBufferLength = WAVEIN_BUFFER_LENGTH;
 wi[i].dwBytesRecorded =  0;
 wi[i].dwUser = 0;
 wi[i].dwFlags = 0;
 wi[i].dwLoops = 0;
 }

waveInOpen(&hwi,
            waveInDevID,
            &waveformatex,
            (DWORD)waveInProc,
            (DWORD)this,
            CALLBACK_FUNCTION|WAVE_FORMAT_DIRECT);


for(i = 0; i < (WAVEIN_NUM_OF_BUFFERS - 1); i++)
 {
 waveInPrepareHeader(hwi,&wi[i],sizeof(wi[i]));
 waveInAddBuffer(hwi,&wi[i],sizeof(wi[i]));
 }

wavein_buffer_id = WAVEIN_NUM_OF_BUFFERS - 1;
wavein_flag = 0;

rxEvent = CreateEvent(NULL, FALSE, FALSE, NULL);

waveInStart(hwi);
}

//=================================================

WAVEIN::~WAVEIN()
{
int i;

waveInStop(hwi);
waveInReset(hwi);

for(i = 0; i < WAVEIN_NUM_OF_BUFFERS; i++) 
 {
 waveInUnprepareHeader(hwi,&wi[i],sizeof(wi[i]));
 delete wi_data[i];
 }

waveInClose(hwi);  

DeleteCriticalSection(&cswi);
CloseHandle(rxEvent);
}

//========================================

char * WAVEIN::GetBuffer(void)  // NULL = not ready, otherwise returns pointer
{
char *buffer;
int i;

EnterCriticalSection(&cswi);
i = wavein_flag;
LeaveCriticalSection(&cswi);
 

if(!i) buffer = NULL;
else
 {
 EnterCriticalSection(&cswi);
 wavein_flag--;
 LeaveCriticalSection(&cswi);
 i = wavein_buffer_id;
 
 wavein_buffer_id++;
 if(wavein_buffer_id >= WAVEIN_NUM_OF_BUFFERS) wavein_buffer_id = 0;

 // Switch buffers
 
 wi[i].lpData = wi_data[i];
 wi[i].dwBufferLength = WAVEIN_BUFFER_LENGTH;
 wi[i].dwBytesRecorded =  0;
 wi[i].dwUser = 0;
 wi[i].dwFlags = 0;
 wi[i].dwLoops = 0;
 
 waveInPrepareHeader(hwi,&wi[i],sizeof(wi[i]));
 waveInAddBuffer(hwi,&wi[i],sizeof(wi[i]));

 waveInUnprepareHeader(hwi,&wi[wavein_buffer_id],sizeof(wi[wavein_buffer_id]));
 buffer = wi_data[wavein_buffer_id];
 } 

return buffer;
}


//================================== WAVEOUT =======================================================

void CALLBACK waveOutProc(HWAVEOUT hwo,UINT uMsg,DWORD dwInstance,DWORD dwParam1,DWORD dwParam2)
 {
 WAVEOUT *waveout;

 if(uMsg == WOM_DONE)
  {
  waveout = (WAVEOUT *)dwInstance;
  
  EnterCriticalSection(&waveout->cswo);
  waveout->waveout_flag++;
  LeaveCriticalSection(&waveout->cswo);
  SetEvent(waveout->txEvent);
  }
 }

//==============================================

WAVEOUT::WAVEOUT(UINT waveOutDevID, UINT channels, int fs, UINT bufferLength)
{
int i;

//============== Initialize Wave Output ==================


InitializeCriticalSection(&cswo); 

waveformatex.wFormatTag = WAVE_FORMAT_PCM;
waveformatex.nChannels  = channels;
waveformatex.nSamplesPerSec  = fs;
waveformatex.nAvgBytesPerSec = (int)(fs*2*channels);
waveformatex.nBlockAlign = 2*channels;
waveformatex.wBitsPerSample = 16;
waveformatex.cbSize = 0;

bufferSizeBytes = bufferLength*2*channels;

for(i=0;i<WAVEOUT_NUM_OF_BUFFERS;i++)
 {
 wh_data[i] = new char[bufferSizeBytes];
 memset(wh_data[i],0,bufferSizeBytes);
 wh[i].lpData = wh_data[i];
 wh[i].dwBufferLength = bufferSizeBytes;
 wh[i].dwBytesRecorded =  0;
 wh[i].dwUser = 0;
 wh[i].dwFlags = 0;
 wh[i].dwLoops = 0;
 }

txEvent = CreateEvent(NULL, FALSE, FALSE, NULL);


waveOutOpen(&hwo,
            waveOutDevID,
            &waveformatex,
            (DWORD)waveOutProc,
            (DWORD)this, // Callbackinstance     
            CALLBACK_FUNCTION|WAVE_FORMAT_DIRECT);

for(i = 0; i < (WAVEOUT_NUM_OF_BUFFERS - 1); i++)
 {
 waveOutPrepareHeader(hwo,&wh[i],sizeof(wh[i]));
 waveOutWrite(hwo,&wh[i],sizeof(wh[i]));
 } 

waveout_buffer_id = WAVEOUT_NUM_OF_BUFFERS - 1;
waveout_flag = 0; 
}
//=================================================

WAVEOUT::~WAVEOUT()
{
int i;

waveOutPause(hwo);
waveOutReset(hwo);

for(i = 0; i < WAVEOUT_NUM_OF_BUFFERS; i++) 
 {
 waveOutUnprepareHeader(hwo,&wh[i],sizeof(wh[i]));
 delete wh_data[i];
 }

waveOutClose(hwo);  

DeleteCriticalSection(&cswo);
CloseHandle(txEvent);
}

//===============================================

char * WAVEOUT::GetBuffer(void)
{
int i;
char *buffer;

EnterCriticalSection(&cswo);
i = waveout_flag;
LeaveCriticalSection(&cswo);

if(!i) buffer = NULL;
else
 {
 EnterCriticalSection(&cswo);
 waveout_flag--;
 LeaveCriticalSection(&cswo);
 
 i = waveout_buffer_id;
 
 waveout_buffer_id++;
 if(waveout_buffer_id >= WAVEOUT_NUM_OF_BUFFERS) waveout_buffer_id = 0;

 
 wh[i].lpData = wh_data[i];
 wh[i].dwBufferLength = bufferSizeBytes;
 wh[i].dwBytesRecorded =  0;
 wh[i].dwUser = 0;
 wh[i].dwFlags = 0;
 wh[i].dwLoops = 0;
 
 waveOutPrepareHeader(hwo,&wh[i],sizeof(wh[i]));
 waveOutWrite(hwo,&wh[i],sizeof(wh[i]));

 waveOutUnprepareHeader(hwo,&wh[waveout_buffer_id],sizeof(wh[waveout_buffer_id]));
 buffer = wh_data[waveout_buffer_id];
 }

 
return buffer;
}
