#ifndef MONTGOMERY_H_INCLUDED
#define MONTGOMERY_H_INCLUDED

#include "types.h"
#include "large_math.h"
#include "fast_mul.h"
#include "fast_modulo.h"
#include "inverse.h"

//---------------------
// Montgomery multiplication

class MONPRO
{
u32 len;
u32 *t;
u32 *u;
u32 *acc;
u32 *n;
u32 *n_;
FAST_MUL *fast_mul;
INVERSE_MODULO *inverse_modulo;


public:
        MONPRO(u32 length_x32);
        ~MONPRO();
        void monpro_init(u32 const *p); // Initialize all constants
        void montgomerize(u32 *x); // Convert to montgomery form 
        void demontgomerize(u32 *x); // Convert back to numbers
        void monpro(u32 *result, u32 *const a, u32 *const b); // result = (ab) mod P
        void reset();
        

};



class MONTGOMERY_POW_MOD
{
u32 len;    
u32 *tmp0;
u32 *tmp1;
u32 *tmp2;
u32 *tmp3;

u32 *tmp_result;
u32 *pwr_x;
u32 *n_;
u32 const *n;

INVERSE_MODULO *inverse_modulo;

//- monpro variables ---
u32 *t;
u32 *u;


void monpro(u32 *result, u32 const* a, u32 const* b);

//-------
// precalculated constants for known p
// (faster calculation)
//
int p_is_precalculated;
u32 *p_precalculated;



public: 
    MONTGOMERY_POW_MOD(const u32 length_x32);
    ~MONTGOMERY_POW_MOD();
     void multi_pow_mod(u32 *result, const u32 *x, const u32 *y, const u32 *p);   // (x ^ y) modulo p; x must be less then p, p must be odd

     FAST_MUL *fast_mul; // Made it public; Used in Miller-Rabin test
     FAST_MODULO *fm; // Made it public; Used in Miller-Rabin test
     void reset();

};

#endif

